clear; clf; close all; clc;

%Load visible image (KAIST site)
visible = im2double(imread('\Proj\Visible\I01840.jpg'));
%Load LWIR image
lwir = im2double(imread('\Proj\lwir\I01840.jpg')); 

% Downloaded KAIST images are normalized(visible and lwri)
%I = (I-min(I(:))) ./ (max(I(:))-min(I(:)));
lwir = lwir(:,:,1);

subplot(2,2,1);
imshow(visible(1:end-10,1:end-10,:));
%imshow(I.^(1/2.2));
title('visible');

subplot(2,2,2);
imshow(lwir(1:end-10,1:end-10,:));
%imshow(I.^(1/2.2));
title('lwir original');

%Simulated lwir image downsample, ratio 1:4
%Matlab imresize automatically applies antialiasing, low pass filter for 
%Down sampling lower than 1 ratio
lwir_down = imresize(lwir,0.25,'bilinear');
subplot(2,2,3);
imshow(lwir_down(1:end-10,1:end-10,:));
title('lwir down sampled');

%bilinear upsample the low resolution lwir image
lwir_upsamp_base = imresize(lwir_down,4,'bilinear');

%%%%%
%Iintensity = (20*visible(:,:,1)+40*visible(:,:,2)+visible(:,:,3)) ./61; 
Iintensity = visible(:,:,1);

% compute the log10 intensity: L = log10(I) 
L = log10(Iintensity);

%bilateral filter parameters for visible image
averageFilterRadius = 5;
sigmaSpatial = 3.;
sigmaIntensity = 0.2;

%Obtain visible base layer, low frequency information, from visible image 
visible_base = bilateral(L, averageFilterRadius, sigmaSpatial, sigmaIntensity); 

%%%%
%Compute the visible detail layer
visible_detail = L - visible_base;
%%%%%

%Combined visible_detail and lwri_base images
figure(2); imagesc(lwir_upsamp_base); axis image; title('lwir upsample base layer'); colormap gray
figure(3); imagesc(visible_detail); axis image; title('visible detail layer'); colormap gray
% Apply an offset and a scale to the base: B' = (B - o) * s 
% The offset is such that the maximum intensity of the base is 1. Since the values are in the log domain, o = max(B).
% The scale is set so that the output base has dR stops of dynamic range, i.e., s = dR / (max(B) - min(B)). 


s = log10(40) / (max(lwir_upsamp_base(:)) - min(lwir_upsamp_base(:)));
BB = (lwir_upsamp_base - max(lwir_upsamp_base(:))).*s;
figure(4); imagesc(BB); axis image; title('scaled lwir upsample base layer'); colormap gray

% Reconstruct the log intensity:
O = 10.^(BB + visible_detail); 

% chrominance channels 
Ichrominance = visible ./ repmat(Iintensity, [1 1 3]);
Ichrominance(isnan(Ichrominance))=0;

% convert back to RGB and apply gamma correction (2.2)
Itonemapped3 = repmat(O, [1 1 3]) .* Ichrominance;
Itonemapped3(Itonemapped3>1) = 1;
Itonemapped3 = Itonemapped3.^(1/2.2);

%convert result image to gray scale
lwir_res_gray = rgb2gray(Itonemapped3); 

figure(1)
subplot(2,2,4);
%imshow(Itonemapped3);
imshow(lwir_res_gray(1:end-10,1:end-10,:));
title('lwir upsampled bilateral');
%imwrite(Itonemapped3, 'I_bilateral_color.png');

%%%%%

m = 512; n = 640;
MSE_up = 1/n/m*sum(sum((lwir_upsamp_base - lwir_res_gray).^2))
PSNR_up = 10*log10(max(lwir_upsamp_base(:).^2)/MSE_up)
%'title(['Image res, PSNR = ' num2str(PSNR) ' dB'], 'FontSize', 5);

MSE = 1/n/m*sum(sum((lwir - lwir_res_gray).^2))
PSNR = 10*log10(max(lwir(:).^2)/MSE)
%'title(['Image res, PSNR = ' num2str(PSNR) ' dB'], 'FontSize', 5);

%%%%%
