function Ifilt = bilateral(I, averageFilterRadius, sigmaSpatial, sigmaIntensity)

    % pad image to reduce boundary artifacts
    I = padarray(I,[averageFilterRadius averageFilterRadius],'symmetric');

    % initialize filtered image as 0
    Ifilt = zeros(size(I));
        
    % smoothing kernel
    spatialKernel = fspecial('gaussian', [2*averageFilterRadius+1 2*averageFilterRadius+1], sigmaSpatial);
        
    for ky= 1+averageFilterRadius:size(I,1)-averageFilterRadius
        for kx= 1+averageFilterRadius:size(I,2)-averageFilterRadius
            
            % extract current pixel
            currentPixel = I(ky,kx,:);
                            
            % accumulated normalization factor
            normalizationFactor = zeros([1 1 size(I,3)]);
            
            for ii=-averageFilterRadius:averageFilterRadius
                for jj=-averageFilterRadius:averageFilterRadius
                    
                    yy = ky+ii;
                    xx = kx+jj;
                    
                    if  (yy>0) && (yy<=size(I,1)) ...
                            && (xx>0) && (xx<=size(I,2)) ...
                            
                        % extract other pixel
                        otherPixel = I(yy,xx,:);                        
                        
                        % intensity distance
                        %d = (currentPixel-otherPixel)^2 ;
                        d = (currentPixel-otherPixel).^2 ;
                        
                        % compute weight
                        weight = spatialKernel(ii+averageFilterRadius+1,jj+averageFilterRadius+1) .* exp( -d ./ (sigmaIntensity^2) );                        
                        %weight = spatialKernel(ii+averageFilterRadius+1,jj+averageFilterRadius+1);                        
                        
                        % add to normalization factor
                        normalizationFactor = normalizationFactor + weight;

                        % add to filtered image
                        Ifilt(ky,kx,:) = Ifilt(ky,kx,:) + weight .* I(yy,xx,:);                                
                    end

                end
            end

            % normalize pixel value
            Ifilt(ky,kx,:) = Ifilt(ky,kx,:) ./ normalizationFactor;

        end
    end
    
    % crop black boundary
    Ifilt = Ifilt(averageFilterRadius+1:end-averageFilterRadius, averageFilterRadius+1:end-averageFilterRadius,:);
    end
